package com.care.common.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.care.common.entity.*;
import com.care.common.enums.*;
import com.care.common.mapper.CareEventOrderMapper;
import com.care.common.service.*;
import com.care.common.vo.order.EventStaVO;
import org.apache.commons.compress.utils.Lists;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * 事件定单表(CareEventOrder)表服务实现类
 *
 * @author makejava
 * @since 2021-05-21 00:08:29
 */
@Service
public class CareEventOrderServiceImpl extends ServiceImpl<CareEventOrderMapper, CareEventOrder> implements CareEventOrderService {

    @Resource
    private CareEventOrderHandleHisService careEventOrderHandleHisService;
    @Resource
    private CareHouseContactService careHouseContactService;
    @Resource
    private CareOlderService careOlderService;
    @Resource
    private CareEventOrderOlderService careEventOrderOlderService;
    @Resource
    private CareEventOrderChambService careEventOrderChambService;
    @Resource
    private CareEventOrderContactStatusService careEventOrderContactStatusService;
    @Resource
    private CareHouseService careHouseService;

    @Resource
    private CareSysUserService careSysUserService;
    /**
     * 统计事件数量
     *
     * @param orgId
     * @param stationId
     * @return
     */
    @Override
    public EventStaVO statOrder(Long orgId, Long stationId) {
        return this.baseMapper.statOrder(orgId,stationId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveOrder(CareEventOrder order){
        this.baseMapper.insert(order);
        CareEventOrderHandleHis his = new CareEventOrderHandleHis();
        his.setOrgId(order.getOrgId());
        his.setStationId(order.getStationId());
        his.setOrderId(order.getId());
        his.setHouseId(order.getHouseId());
        if (OrderTypeEnum.ZHU_DONG_HU_JIAO.getValue().equals(order.getOrderType()) || OrderTypeEnum.HU_WAI_HU_JIAO.getValue().equals(order.getOrderType())){
            his.setLogType(LogTypeEnum.OLDER_EVENT.getValue());
            his.setLogObjectId(order.getOlderId());
            his.setLogObjectName(order.getOlderName());
            his.setLogResult(OrderTypeEnum.getCodeToName(order.getOrderType()));
            his.setRelationTypeDesc(RelationTypeEnum.OLDER.getName());
        }else{
            his.setLogType(LogTypeEnum.DEV_EVENT.getValue());
            his.setLogObjectId(order.getDevId());
            his.setLogObjectName(order.getDevName());
            his.setLogResult("发生"+OrderTypeEnum.getCodeToName(order.getOrderType()));
            his.setRelationTypeDesc(RelationTypeEnum.HOUSE.getName());
        }
        his.setCreateTime(DateUtil.date());
        addOlderToOrderOlder(order);
        addChamberlainToOrder(order);
        addContactToOrderContact(order);
        this.careEventOrderHandleHisService.save(his);
        return true;
    }

    /**
     * 将老人加入到 工单中
     * @param order
     */
    private void addOlderToOrderOlder(CareEventOrder order){
        QueryWrapper<CareOlder> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareOlder::getHouseId,order.getHouseId());
        List<CareOlder> olders = this.careOlderService.list(queryWrapper);
        if (CollUtil.isNotEmpty(olders)){
            List<CareEventOrderOlder>  ceoos = Lists.newArrayList();
            olders.forEach(item -> {
                CareEventOrderOlder ceoo = new CareEventOrderOlder();
                BeanUtil.copyProperties(item,ceoo);
                ceoo.setOrderId(order.getId());
                ceoo.setOlderId(item.getId());
                if (order.getOlderId() == item.getId()){
                    //主动呼叫
                    ceoo.setIsZhudong(1);
                }else{
                    ceoo.setIsZhudong(0);
                }
                ceoo.setStatus(OrderOlderStatusEnum.WEI_LIAN_XI.getValue());
                ceoo.setCreateTime(DateUtil.date());
                ceoos.add(ceoo);
            });
            this.careEventOrderOlderService.saveBatch(ceoos);
        }
    }

    /**
     * 将管家加入到联系人当中
     * @param order
     */
    private void addChamberlainToOrder(CareEventOrder order){
        CareHouse house = this.careHouseService.getById(order.getHouseId());
        if (house != null){
            CareSysUser user = this.careSysUserService.getById(house.getChambId());
            if (user != null){
                CareEventOrderChamb chamb = new CareEventOrderChamb();
                chamb.setOrgId(user.getOrgId());
                chamb.setStationId(user.getStationId());
                chamb.setOrderId(order.getId());
                chamb.setChambId(user.getId());
                chamb.setChambName(user.getName());
                chamb.setPhone(user.getPhone());
                chamb.setStatus(ChambOrderStatusEnum.TODO.getValue());
                chamb.setHouseId(house.getId());
                chamb.setCreateTime(DateUtil.date());
                chamb.setModifyTime(DateUtil.date());
                this.careEventOrderChambService.save(chamb);
            }
        }
    }

    /**
     * 将紧急联系人加入到联系人当中
     * @param order
     */
    private void addContactToOrderContact(CareEventOrder order){
        List<CareHouseContact> cons = this.careHouseContactService.queryContactByHouseId(order.getHouseId());
        if (CollUtil.isNotEmpty(cons)){
            List<CareEventOrderContactStatus> orderContactList = Lists.newArrayList();
            cons.forEach(item ->{
                CareEventOrderContactStatus orderContactStatus = new CareEventOrderContactStatus();
                orderContactStatus.setOrgId(order.getOrgId());
                orderContactStatus.setStationId(order.getStationId());
                orderContactStatus.setOrderId(order.getId());
                orderContactStatus.setHouseId(order.getHouseId());
                orderContactStatus.setContactRole(UserRoleEnum.CONTACT.getValue());
                orderContactStatus.setContactId(item.getId());
                orderContactStatus.setStatus(ContactorStatusEnum.WEI_LIAN_XI.getValue());
                orderContactStatus.setContactName(item.getName());
                orderContactStatus.setContactPhone(item.getPhone());
                orderContactStatus.setLongitude(item.getLongitude());
                orderContactStatus.setLatitude(item.getLatitude());
                orderContactStatus.setRelationTypeDesc(RelationTypeEnum.getCodeToName(item.getRelationType()));
                orderContactStatus.setContactLevel(item.getContactLevel());
                orderContactStatus.setCreateTime(DateUtil.date());
                orderContactStatus.setModifyTime(DateUtil.date());
                orderContactList.add(orderContactStatus);
            });
            this.careEventOrderContactStatusService.saveBatch(orderContactList);
        }
    }
}